@@ -7,13 +7,6 @@
 
 #include <QtCore/QVariant>
 
-void Quotient::_impl::warnUnknownEnumValue(const QString& stringValue,
-                                           const char* enumTypeName)
-{
-    qWarning(EVENTS).noquote()
-        << "Unknown" << enumTypeName << "value:" << stringValue;
-}
-
 void Quotient::_impl::reportEnumOutOfBounds(uint32_t v, const char* enumTypeName)
 {
     qCritical(MAIN).noquote()
