@@ -29,9 +29,11 @@ public:
     //! with a `{` as this character is used to determine
     //! if the filter provided is inline JSON or a previously
     //! declared filter by homeservers on some APIs.
-    QString filterId() const { return loadFromJson<QString>("filter_id"_ls); }
+    QString filterId() const { return loadFromJson<QString>("filter_id"_L1); }
 };
 
+inline auto collectResponse(const DefineFilterJob* job) { return job->filterId(); }
+
 //! \brief Download a filter
 class QUOTIENT_API GetFilterJob : public BaseJob {
 public:
@@ -46,7 +48,8 @@ public:
     //!
     //! This function can be used when a URL for GetFilterJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& userId, const QString& filterId);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& userId,
+                               const QString& filterId);
 
     // Result properties
 
@@ -54,4 +57,6 @@ public:
     Filter filter() const { return fromJson<Filter>(jsonData()); }
 };
 
+inline auto collectResponse(const GetFilterJob* job) { return job->filter(); }
+
 } // namespace Quotient
