@@ -7,16 +7,16 @@ using namespace Quotient;
 auto queryToSearch(const QString& nextBatch)
 {
     QUrlQuery _q;
-    addParam<IfNotEmpty>(_q, QStringLiteral("next_batch"), nextBatch);
+    addParam<IfNotEmpty>(_q, u"next_batch"_s, nextBatch);
     return _q;
 }
 
 SearchJob::SearchJob(const Categories& searchCategories, const QString& nextBatch)
-    : BaseJob(HttpVerb::Post, QStringLiteral("SearchJob"),
-              makePath("/_matrix/client/v3", "/search"), queryToSearch(nextBatch))
+    : BaseJob(HttpVerb::Post, u"SearchJob"_s, makePath("/_matrix/client/v3", "/search"),
+              queryToSearch(nextBatch))
 {
     QJsonObject _dataJson;
-    addParam<>(_dataJson, QStringLiteral("search_categories"), searchCategories);
+    addParam(_dataJson, "search_categories"_L1, searchCategories);
     setRequestData({ _dataJson });
-    addExpectedKey("search_categories");
+    addExpectedKey(u"search_categories"_s);
 }
