@@ -4,46 +4,45 @@
 
 using namespace Quotient;
 
-QUrl GetProtocolsJob::makeRequestUrl(QUrl baseUrl)
+QUrl GetProtocolsJob::makeRequestUrl(const HomeserverData& hsData)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
-                                   makePath("/_matrix/client/v3", "/thirdparty/protocols"));
+    return BaseJob::makeRequestUrl(hsData, makePath("/_matrix/client/v3", "/thirdparty/protocols"));
 }
 
 GetProtocolsJob::GetProtocolsJob()
-    : BaseJob(HttpVerb::Get, QStringLiteral("GetProtocolsJob"),
+    : BaseJob(HttpVerb::Get, u"GetProtocolsJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/protocols"))
 {}
 
-QUrl GetProtocolMetadataJob::makeRequestUrl(QUrl baseUrl, const QString& protocol)
+QUrl GetProtocolMetadataJob::makeRequestUrl(const HomeserverData& hsData, const QString& protocol)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl), makePath("/_matrix/client/v3",
-                                                                "/thirdparty/protocol/", protocol));
+    return BaseJob::makeRequestUrl(hsData, makePath("/_matrix/client/v3", "/thirdparty/protocol/",
+                                                    protocol));
 }
 
 GetProtocolMetadataJob::GetProtocolMetadataJob(const QString& protocol)
-    : BaseJob(HttpVerb::Get, QStringLiteral("GetProtocolMetadataJob"),
+    : BaseJob(HttpVerb::Get, u"GetProtocolMetadataJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/protocol/", protocol))
 {}
 
 auto queryToQueryLocationByProtocol(const QString& searchFields)
 {
     QUrlQuery _q;
-    addParam<IfNotEmpty>(_q, QStringLiteral("searchFields"), searchFields);
+    addParam<IfNotEmpty>(_q, u"searchFields"_s, searchFields);
     return _q;
 }
 
-QUrl QueryLocationByProtocolJob::makeRequestUrl(QUrl baseUrl, const QString& protocol,
-                                                const QString& searchFields)
+QUrl QueryLocationByProtocolJob::makeRequestUrl(const HomeserverData& hsData,
+                                                const QString& protocol, const QString& searchFields)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
+    return BaseJob::makeRequestUrl(hsData,
                                    makePath("/_matrix/client/v3", "/thirdparty/location/", protocol),
                                    queryToQueryLocationByProtocol(searchFields));
 }
 
 QueryLocationByProtocolJob::QueryLocationByProtocolJob(const QString& protocol,
                                                        const QString& searchFields)
-    : BaseJob(HttpVerb::Get, QStringLiteral("QueryLocationByProtocolJob"),
+    : BaseJob(HttpVerb::Get, u"QueryLocationByProtocolJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/location/", protocol),
               queryToQueryLocationByProtocol(searchFields))
 {}
@@ -51,21 +50,21 @@ QueryLocationByProtocolJob::QueryLocationByProtocolJob(const QString& protocol,
 auto queryToQueryUserByProtocol(const QHash<QString, QString>& fields)
 {
     QUrlQuery _q;
-    addParam<IfNotEmpty>(_q, QStringLiteral("fields"), fields);
+    addParam<IfNotEmpty>(_q, u"fields"_s, fields);
     return _q;
 }
 
-QUrl QueryUserByProtocolJob::makeRequestUrl(QUrl baseUrl, const QString& protocol,
+QUrl QueryUserByProtocolJob::makeRequestUrl(const HomeserverData& hsData, const QString& protocol,
                                             const QHash<QString, QString>& fields)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
+    return BaseJob::makeRequestUrl(hsData,
                                    makePath("/_matrix/client/v3", "/thirdparty/user/", protocol),
                                    queryToQueryUserByProtocol(fields));
 }
 
 QueryUserByProtocolJob::QueryUserByProtocolJob(const QString& protocol,
                                                const QHash<QString, QString>& fields)
-    : BaseJob(HttpVerb::Get, QStringLiteral("QueryUserByProtocolJob"),
+    : BaseJob(HttpVerb::Get, u"QueryUserByProtocolJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/user/", protocol),
               queryToQueryUserByProtocol(fields))
 {}
@@ -73,19 +72,18 @@ QueryUserByProtocolJob::QueryUserByProtocolJob(const QString& protocol,
 auto queryToQueryLocationByAlias(const QString& alias)
 {
     QUrlQuery _q;
-    addParam<>(_q, QStringLiteral("alias"), alias);
+    addParam(_q, u"alias"_s, alias);
     return _q;
 }
 
-QUrl QueryLocationByAliasJob::makeRequestUrl(QUrl baseUrl, const QString& alias)
+QUrl QueryLocationByAliasJob::makeRequestUrl(const HomeserverData& hsData, const QString& alias)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
-                                   makePath("/_matrix/client/v3", "/thirdparty/location"),
+    return BaseJob::makeRequestUrl(hsData, makePath("/_matrix/client/v3", "/thirdparty/location"),
                                    queryToQueryLocationByAlias(alias));
 }
 
 QueryLocationByAliasJob::QueryLocationByAliasJob(const QString& alias)
-    : BaseJob(HttpVerb::Get, QStringLiteral("QueryLocationByAliasJob"),
+    : BaseJob(HttpVerb::Get, u"QueryLocationByAliasJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/location"),
               queryToQueryLocationByAlias(alias))
 {}
@@ -93,18 +91,17 @@ QueryLocationByAliasJob::QueryLocationByAliasJob(const QString& alias)
 auto queryToQueryUserByID(const QString& userid)
 {
     QUrlQuery _q;
-    addParam<>(_q, QStringLiteral("userid"), userid);
+    addParam(_q, u"userid"_s, userid);
     return _q;
 }
 
-QUrl QueryUserByIDJob::makeRequestUrl(QUrl baseUrl, const QString& userid)
+QUrl QueryUserByIDJob::makeRequestUrl(const HomeserverData& hsData, const QString& userid)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
-                                   makePath("/_matrix/client/v3", "/thirdparty/user"),
+    return BaseJob::makeRequestUrl(hsData, makePath("/_matrix/client/v3", "/thirdparty/user"),
                                    queryToQueryUserByID(userid));
 }
 
 QueryUserByIDJob::QueryUserByIDJob(const QString& userid)
-    : BaseJob(HttpVerb::Get, QStringLiteral("QueryUserByIDJob"),
+    : BaseJob(HttpVerb::Get, u"QueryUserByIDJob"_s,
               makePath("/_matrix/client/v3", "/thirdparty/user"), queryToQueryUserByID(userid))
 {}
