@@ -10,16 +10,10 @@ using namespace Quotient;
 
 void PendingEventItem::setFileUploaded(const FileSourceInfo& uploadedFileData)
 {
-    // TODO: eventually we might introduce hasFileContent to RoomEvent,
-    // and unify the code below.
-    if (auto* rme = getAs<RoomMessageEvent>()) {
-        Q_ASSERT(rme->hasFileContent());
-        rme->editContent([&uploadedFileData](EventContent::TypedBase& ec) {
-            ec.fileInfo()->source = uploadedFileData;
-        });
-    }
+    if (auto* rme = getAs<RoomMessageEvent>())
+        rme->updateFileSourceInfo(uploadedFileData);
+
     if (auto* rae = getAs<RoomAvatarEvent>()) {
-        Q_ASSERT(rae->content().fileInfo());
         rae->editContent([&uploadedFileData](EventContent::FileInfo& fi) {
             fi.source = uploadedFileData;
         });
@@ -29,4 +23,4 @@ void PendingEventItem::setFileUploaded(const FileSourceInfo& uploadedFileData)
 
 // Not exactly sure why but this helps with the linker not finding
 // Quotient::EventStatus::staticMetaObject when building Quaternion
-#include "moc_eventitem.cpp"
+#include "moc_eventitem.cpp" // NOLINT(bugprone-suspicious-include)
