@@ -133,11 +133,11 @@ TextContent::TextContent(const QJsonObject& json)
     const auto relatesTo = fromJson<std::optional<EventRelation>>(json[RelatesToKey]);
 
     const auto actualJson = relatesTo.has_value() && relatesTo->type == EventRelation::ReplacementType
-                                ? json.value("m.new_content"_L1).toObject()
+                                ? json.value(NewContentKey).toObject()
                                 : json;
     // Special-casing the custom matrix.org's (actually, Element's) way
     // of sending HTML messages.
-    if (actualJson["format"_L1].toString() == HtmlContentTypeId) {
+    if (actualJson[FormatKey].toString() == HtmlContentTypeId) {
         mimeType = HtmlMimeType;
         body = actualJson[FormattedBodyKey].toString();
     } else {
