@@ -4,19 +4,23 @@
 
 #include "basejob.h"
 
-#include <Quotient/connectiondata.h>
+#include "../logging_categories_p.h"
 
-#include <QtCore/QRegularExpression>
-#include <QtCore/QTimer>
+#include "../connectiondata.h"
+#include "../networkaccessmanager.h"
+
+#include <QtCore/QLibraryInfo>
 #include <QtCore/QMetaEnum>
 #include <QtCore/QPointer>
-#include <QtNetwork/QNetworkAccessManager>
+#include <QtCore/QRegularExpression>
+#include <QtCore/QTimer>
 #include <QtNetwork/QNetworkReply>
 #include <QtNetwork/QNetworkRequest>
 
+#include <ranges>
+
 using namespace Quotient;
-using std::chrono::seconds, std::chrono::milliseconds;
-using namespace std::chrono_literals;
+using namespace std::chrono;
 
 BaseJob::StatusCode BaseJob::Status::fromHttpCode(int httpCode)
 {
@@ -62,11 +66,6 @@ QDebug BaseJob::Status::dumpToLog(QDebug dbg) const
 
 class Q_DECL_HIDDEN BaseJob::Private {
 public:
-    struct JobTimeoutConfig {
-        seconds jobTimeout;
-        seconds nextRetryInterval;
-    };
-
     // Using an idiom from clang-tidy:
     // http://clang.llvm.org/extra/clang-tidy/checks/modernize-pass-by-value.html
     Private(HttpVerb v, QByteArray endpoint, const QUrlQuery& q,
@@ -91,7 +90,9 @@ public:
         }
     }
 
-    void sendRequest();
+    QNetworkRequest prepareRequest() const;
+    void sendRequest(const QNetworkRequest& req);
+
     /*! \brief Parse the response byte array into JSON
      *
      * This calls QJsonDocument::fromJson() on rawResponse, converts
@@ -117,12 +118,14 @@ public:
     // type QMimeType is of little help with MIME type globs (`text/*` etc.)
     QByteArrayList expectedContentTypes { "application/json" };
 
-    QByteArrayList expectedKeys;
+    QStringList expectedKeys;
 
     // When the QNetworkAccessManager is destroyed it destroys all pending replies.
     // Using QPointer allows us to know when that happend.
     QPointer<QNetworkReply> reply;
 
+    QPromise<void> promise{};
+
     Status status = Unprepared;
     QByteArray rawResponse;
     /// Contains a null document in case of non-JSON body (for a successful
@@ -131,51 +134,35 @@ public:
     /// definition (including an empty JSON object - QJsonObject{});
     /// and QJsonObject in case of an API error.
     QJsonDocument jsonResponse;
-    QUrl errorUrl; //< May contain a URL to help with some errors
+    QUrl errorUrl; //!< May contain a URL to help with some errors
 
-    LoggingCategory logCat = JOBS;
+    QMessageLogger::CategoryFunction logCat = &JOBS;
 
     QTimer timer;
     QTimer retryTimer;
 
-    static constexpr auto errorStrategy = std::to_array<const JobTimeoutConfig>(
-        { { 90s, 5s }, { 90s, 10s }, { 120s, 30s } });
-    int maxRetries = int(errorStrategy.size());
-    int retriesTaken = 0;
+    static inline JobBackoffStrategy defaultBackoffStrategy{ { 45s, 90s, 150s }, { 2s, 5s } };
 
-    [[nodiscard]] const JobTimeoutConfig& getCurrentTimeoutConfig() const
-    {
-        return errorStrategy[std::min(size_t(retriesTaken),
-                                      errorStrategy.size() - 1)];
-    }
+    JobBackoffStrategy backoffStrategy = defaultBackoffStrategy;
+    int retriesTaken = 0;
 
     [[nodiscard]] QString dumpRequest() const
     {
-        static const std::array verbs { "GET"_ls, "PUT"_ls, "POST"_ls,
-                                        "DELETE"_ls };
+        static const std::array verbs { "GET"_L1, "PUT"_L1, "POST"_L1,
+                                        "DELETE"_L1 };
         const auto verbWord = verbs.at(size_t(verb));
         return verbWord % u' '
                % (reply ? reply->url().toString(QUrl::RemoveQuery)
-                        : makeRequestUrl(connection->baseUrl(), apiEndpoint)
-                              .toString());
+                        : makeRequestUrl(connection->homeserverData(), apiEndpoint).toString());
     }
 };
 
-inline bool isHex(QChar c)
-{
-    return c.isDigit() || (c >= u'A' && c <= u'F') || (c >= u'a' && c <= u'f');
-}
-
 QByteArray BaseJob::encodeIfParam(const QString& paramPart)
 {
-    const auto percentIndex = paramPart.indexOf(u'%');
-    if (percentIndex != -1 && paramPart.size() > percentIndex + 2
-        && isHex(paramPart[percentIndex + 1])
-        && isHex(paramPart[percentIndex + 2])) {
-        qCWarning(JOBS)
-            << "Developers, upfront percent-encoding of job parameters is "
-               "deprecated since libQuotient 0.7; the string involved is"
-            << paramPart;
+    if (static const QRegularExpression re{ "%[[:xdigit:]]{2}"_L1 }; paramPart.indexOf(re) != -1) {
+        qCWarning(JOBS) << "Developers, upfront percent-encoding of job parameters is "
+                           "deprecated since libQuotient 0.7; the string involved is"
+                        << paramPart;
         return QUrl(paramPart, QUrl::TolerantMode).toEncoded();
     }
     return QUrl::toPercentEncoding(paramPart);
@@ -211,6 +198,10 @@ QUrl BaseJob::requestUrl() const { return d->reply ? d->reply->url() : QUrl(); }
 
 bool BaseJob::isBackground() const { return d->inBackground; }
 
+QByteArray BaseJob::apiEndpoint() const { return d->apiEndpoint; }
+
+void BaseJob::setApiEndpoint(QByteArray apiEndpoint) { d->apiEndpoint = std::move(apiEndpoint); }
+
 const BaseJob::headers_t& BaseJob::requestHeaders() const
 {
     return d->requestHeaders;
@@ -256,11 +247,11 @@ void BaseJob::setExpectedContentTypes(const QByteArrayList& contentTypes)
     d->expectedContentTypes = contentTypes;
 }
 
-QByteArrayList BaseJob::expectedKeys() const { return d->expectedKeys; }
+QStringList BaseJob::expectedKeys() const { return d->expectedKeys; }
 
-void BaseJob::addExpectedKey(const QByteArray& key) { d->expectedKeys << key; }
+void BaseJob::addExpectedKey(QString key) { d->expectedKeys << std::move(key); }
 
-void BaseJob::setExpectedKeys(const QByteArrayList& keys)
+void BaseJob::setExpectedKeys(const QStringList& keys)
 {
     d->expectedKeys = keys;
 }
@@ -269,7 +260,7 @@ const QNetworkReply* BaseJob::reply() const { return d->reply.data(); }
 
 QNetworkReply* BaseJob::reply() { return d->reply.data(); }
 
-QUrl BaseJob::makeRequestUrl(QUrl baseUrl, const QByteArray& encodedPath,
+QUrl BaseJob::makeRequestUrl(const HomeserverData& hsData, const QByteArray& encodedPath,
                              const QUrlQuery& query)
 {
     // Make sure the added path is relative even if it's not (the official
@@ -279,17 +270,16 @@ QUrl BaseJob::makeRequestUrl(QUrl baseUrl, const QByteArray& encodedPath,
                           QUrl::StrictMode);
     Q_ASSERT_X(pathUrl.isValid(), __FUNCTION__,
                qPrintable(pathUrl.errorString()));
-    baseUrl = baseUrl.resolved(pathUrl);
+    auto baseUrl = hsData.baseUrl.resolved(pathUrl);
     baseUrl.setQuery(query);
     return baseUrl;
 }
 
-void BaseJob::Private::sendRequest()
+QNetworkRequest BaseJob::Private::prepareRequest() const
 {
-    QNetworkRequest req { makeRequestUrl(connection->baseUrl(), apiEndpoint,
-                                         requestQuery) };
+    QNetworkRequest req{ makeRequestUrl(connection->homeserverData(), apiEndpoint, requestQuery) };
     if (!requestHeaders.contains("Content-Type"))
-        req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json"_ls);
+        req.setHeader(QNetworkRequest::ContentTypeHeader, "application/json"_L1);
     if (needsToken)
         req.setRawHeader("Authorization",
                          QByteArray("Bearer ") + connection->accessToken());
@@ -305,7 +295,11 @@ void BaseJob::Private::sendRequest()
     Q_ASSERT(req.url().isValid());
     for (auto it = requestHeaders.cbegin(); it != requestHeaders.cend(); ++it)
         req.setRawHeader(it.key(), it.value());
+    return req;
+}
 
+void BaseJob::Private::sendRequest(const QNetworkRequest& req)
+{
     switch (verb) {
     case HttpVerb::Get:
         reply = connection->nam()->get(req);
@@ -322,7 +316,7 @@ void BaseJob::Private::sendRequest()
     }
 }
 
-void BaseJob::doPrepare() { }
+void BaseJob::doPrepare(const ConnectionData*) { }
 
 void BaseJob::onSentRequest(QNetworkReply*) { }
 
@@ -333,17 +327,18 @@ void BaseJob::initiate(ConnectionData* connData, bool inBackground)
     if (Q_LIKELY(connData && connData->baseUrl().isValid())) {
         d->inBackground = inBackground;
         d->connection = connData;
-        doPrepare();
+        doPrepare(connData);
 
         if (d->needsToken && d->connection->accessToken().isEmpty())
             setStatus(Unauthorised);
         else if ((d->verb == HttpVerb::Post || d->verb == HttpVerb::Put)
             && d->requestData.source()
             && !d->requestData.source()->isReadable()) {
-            setStatus(FileError, "Request data not ready"_ls);
+            setStatus(FileError, "Request data not ready"_L1);
         }
         Q_ASSERT(status().code != Pending); // doPrepare() must NOT set this
         if (Q_LIKELY(status().code == Unprepared)) {
+            d->promise.start();
             d->connection->submit(this);
             return;
         }
@@ -356,7 +351,7 @@ void BaseJob::initiate(ConnectionData* connData, bool inBackground)
         setStatus(IncorrectRequest, tr("Invalid server connection"));
     }
     // The status is no good, finalise
-    QTimer::singleShot(0, this, &BaseJob::finishJob);
+    QMetaObject::invokeMethod(this, &BaseJob::finishJob, Qt::QueuedConnection);
 }
 
 void BaseJob::sendRequest()
@@ -368,9 +363,9 @@ void BaseJob::sendRequest()
         return;
     }
     Q_ASSERT(d->connection && status().code == Pending);
-    d->needsToken |= d->connection->needsToken(objectName());
-    emit aboutToSendRequest();
-    d->sendRequest();
+    auto req = d->prepareRequest();
+    emit aboutToSendRequest(&req);
+    d->sendRequest(req);
     Q_ASSERT(d->reply);
     connect(reply(), &QNetworkReply::finished, this, [this] {
         gotReply();
@@ -405,31 +400,29 @@ void BaseJob::gotReply()
 {
     // Defer actually updating the status until it's finalised
     auto statusSoFar = checkReply(reply());
-    if (statusSoFar.good()
-        && d->expectedContentTypes == QByteArrayList { "application/json" }) //
-    {
+    if (statusSoFar.good() && d->expectedContentTypes == QByteArrayList{ "application/json"_ba }) {
         d->rawResponse = reply()->readAll();
         statusSoFar = d->parseJson();
-        if (statusSoFar.good() && !expectedKeys().empty()) {
-            const auto& responseObject = jsonData();
-            QByteArrayList missingKeys;
-            for (const auto& k: expectedKeys())
-                if (!responseObject.contains(QString::fromLatin1(k)))
-                    missingKeys.push_back(k);
-            if (!missingKeys.empty())
+        if (statusSoFar.good()) {
+            auto filteredView =
+                std::views::filter(expectedKeys(), [responseObject = jsonData()](const QString& k) {
+                    return !responseObject.contains(k);
+                });
+            if (const auto missingKeys =
+                    QStringList(filteredView.begin(), filteredView.end()).join(u',');
+                !missingKeys.isEmpty()) [[unlikely]]
                 statusSoFar = { IncorrectResponse,
-                                tr("Required JSON keys missing: ")
-                                    + QString::fromLatin1(missingKeys.join()) };
+                                tr("Required JSON keys missing: ") + missingKeys };
         }
         setStatus(statusSoFar);
         if (!status().good()) // Bad JSON in a "good" reply: bail out
             return;
+        // If the endpoint expects anything else than just (API-related) JSON
+        // reply()->readAll() is not performed and the whole reply processing
+        // is left to derived job classes: they may read it piecemeal or customise
+        // per content type in prepareResult(), or even have read it already
+        // (see, e.g., DownloadFileJob).
     }
-    // If the endpoint expects anything else than just (API-related) JSON
-    // reply()->readAll() is not performed and the whole reply processing
-    // is left to derived job classes: they may read it piecemeal or customise
-    // per content type in prepareResult(), or even have read it already
-    // (see, e.g., DownloadFileJob).
     if (statusSoFar.good()) {
         setStatus(prepareResult());
         return;
@@ -454,10 +447,10 @@ bool checkContentType(const QByteArray& type, const QByteArrayList& patterns)
             return true;
 
         auto patternParts = pattern.split('/');
-        Q_ASSERT_X(patternParts.size() <= 2, __FUNCTION__,
-                   qPrintable(
-                       "BaseJob: Expected content type should have up to two /-separated parts; violating pattern: "_ls
-                       + QString::fromLatin1(pattern)));
+        if (QUO_ALARM_X(patternParts.size() > 2, "Expected content type should have up to two "
+                                                 "parts separated by `/`; violating pattern: "
+                                                     % pattern))
+            return false;
 
         if (ctype.split('/').front() == patternParts.front()
             && patternParts.back() == "*")
@@ -487,7 +480,7 @@ BaseJob::Status BaseJob::checkReply(const QNetworkReply* reply) const
         if (!checkContentType(reply->rawHeader("Content-Type"),
                               d->expectedContentTypes))
             return { UnexpectedResponseTypeWarning,
-                     "Unexpected content type of the response"_ls };
+                     "Unexpected content type of the response"_L1 };
         return NoError;
     }
     if (reply->isFinished())
@@ -515,10 +508,10 @@ BaseJob::Status BaseJob::prepareError(Status currentStatus)
     // a valid JSON object - or an empty object otherwise (in which case most
     // of if's below will fall through retaining the current status)
     const auto& errorJson = jsonData();
-    const auto errCode = errorJson.value("errcode"_ls).toString();
-    if (error() == TooManyRequests || errCode == "M_LIMIT_EXCEEDED"_ls) {
+    const auto errCode = errorJson.value("errcode"_L1).toString();
+    if (error() == TooManyRequests || errCode == "M_LIMIT_EXCEEDED"_L1) {
         QString msg = tr("Too many requests");
-        int64_t retryAfterMs = errorJson.value("retry_after_ms"_ls).toInt(-1);
+        int64_t retryAfterMs = errorJson.value("retry_after_ms"_L1).toInt(-1);
         if (retryAfterMs >= 0)
             msg += tr(", next retry advised after %1 ms").arg(retryAfterMs);
         else // We still have to figure some reasonable interval
@@ -529,36 +522,40 @@ BaseJob::Status BaseJob::prepareError(Status currentStatus)
         return { TooManyRequests, msg };
     }
 
-    if (errCode == "M_CONSENT_NOT_GIVEN"_ls) {
-        d->errorUrl = QUrl(errorJson.value("consent_uri"_ls).toString());
+    if (errCode == "M_CONSENT_NOT_GIVEN"_L1) {
+        d->errorUrl = QUrl(errorJson.value("consent_uri"_L1).toString());
         return { UserConsentRequired };
     }
-    if (errCode == "M_UNSUPPORTED_ROOM_VERSION"_ls
-        || errCode == "M_INCOMPATIBLE_ROOM_VERSION"_ls)
+    if (errCode == "M_UNSUPPORTED_ROOM_VERSION"_L1
+        || errCode == "M_INCOMPATIBLE_ROOM_VERSION"_L1)
         return { UnsupportedRoomVersion,
-                 errorJson.contains("room_version"_ls)
+                 errorJson.contains("room_version"_L1)
                      ? tr("Requested room version: %1")
-                           .arg(errorJson.value("room_version"_ls).toString())
-                     : errorJson.value("error"_ls).toString() };
-    if (errCode == "M_CANNOT_LEAVE_SERVER_NOTICE_ROOM"_ls)
+                           .arg(errorJson.value("room_version"_L1).toString())
+                     : errorJson.value("error"_L1).toString() };
+    if (errCode == "M_CANNOT_LEAVE_SERVER_NOTICE_ROOM"_L1)
         return { CannotLeaveRoom,
                  tr("It's not allowed to leave a server notices room") };
-    if (errCode == "M_USER_DEACTIVATED"_ls)
-        return { UserDeactivated };
+
+    if (errCode == "M_USER_DEACTIVATED"_L1)
+        currentStatus = { UserDeactivated };
+    if (errCode == "M_ACCOUNT_LOCKED"_L1)
+        currentStatus = { AccountLocked };
 
     // Not localisable on the client side
-    if (errorJson.contains("error"_ls)) // Keep the code, update the message
-        return { currentStatus.code, errorJson.value("error"_ls).toString() };
+    if (errorJson.contains("error"_L1)) // Keep the code, update the message
+        return { currentStatus.code, errorJson.value("error"_L1).toString() };
 
     return currentStatus; // The error payload is not recognised
 }
 
-QJsonValue BaseJob::takeValueFromJson(const QString& key)
+QJsonValue BaseJob::takeValueFromJson(QAnyStringView key)
 {
     if (!d->jsonResponse.isObject())
         return QJsonValue::Undefined;
     auto o = d->jsonResponse.object();
-    auto v = o.take(key);
+    auto v = key.visit(Overloads{ [&o](QUtf8StringView k) { return o.take(k.utf8()); },
+                                  [&o](auto k) { return o.take(k); } });
     d->jsonResponse.setObject(o);
     return v;
 }
@@ -587,31 +584,19 @@ void BaseJob::finishJob()
         emit rateLimited();
         d->connection->submit(this);
         return;
-    case Unauthorised:
-        if (!d->needsToken && !d->connection->accessToken().isEmpty()) {
-            // Rerun with access token (extension of the spec while
-            // https://github.com/matrix-org/matrix-doc/issues/701 is pending)
-            d->connection->setNeedsToken(objectName());
-            qCWarning(d->logCat) << this << "re-running with authentication";
-            emit retryScheduled(d->retriesTaken, 0);
-            d->connection->submit(this);
-            return;
-        }
-        break;
     case NetworkError:
     case IncorrectResponse:
     case Timeout:
-        if (d->retriesTaken < d->maxRetries) {
+        if (!d->backoffStrategy.maxRetries || d->retriesTaken < *d->backoffStrategy.maxRetries) {
             // TODO: The whole retrying thing should be put to
             // Connection(Manager) otherwise independently retrying jobs make a
             // bit of notification storm towards the UI.
-            const seconds retryIn = error() == Timeout ? 0s
-                                                       : getNextRetryInterval();
+            const auto retryIn = error() == Timeout ? 0s : getNextRetryInterval();
             ++d->retriesTaken;
             qCWarning(d->logCat).nospace()
                 << this << ": retry #" << d->retriesTaken << " in "
                 << retryIn.count() << " s";
-            setStatus(Pending, "Pending retry"_ls);
+            setStatus(Pending, "Pending retry"_L1);
             d->retryTimer.start(retryIn);
             emit retryScheduled(d->retriesTaken, milliseconds(retryIn).count());
             return;
@@ -625,6 +610,7 @@ void BaseJob::finishJob()
     // Notify those interested in any completion of the job including abandon()
     emit finished(this);
 
+    d->promise.finish();
     emit result(this); // abandon() doesn't emit this
     if (error())
         emit failure(this);
@@ -634,9 +620,15 @@ void BaseJob::finishJob()
     deleteLater();
 }
 
-seconds BaseJob::getCurrentTimeout() const
+inline auto atOrLast(const auto& values, auto index)
 {
-    return d->getCurrentTimeoutConfig().jobTimeout;
+    QUO_CHECK(!values.empty());
+    return index < values.size() ? values[index] : values.back();
+}
+
+JobBackoffStrategy::duration_t BaseJob::getCurrentTimeout() const
+{
+    return atOrLast(d->backoffStrategy.jobTimeouts, d->retriesTaken);
 }
 
 BaseJob::duration_ms_t BaseJob::getCurrentTimeoutMs() const
@@ -644,9 +636,9 @@ BaseJob::duration_ms_t BaseJob::getCurrentTimeoutMs() const
     return milliseconds(getCurrentTimeout()).count();
 }
 
-seconds BaseJob::getNextRetryInterval() const
+JobBackoffStrategy::duration_t BaseJob::getNextRetryInterval() const
 {
-    return d->getCurrentTimeoutConfig().nextRetryInterval;
+    return atOrLast(d->backoffStrategy.nextRetryIntervals, d->retriesTaken);
 }
 
 BaseJob::duration_ms_t BaseJob::getNextRetryMs() const
@@ -665,11 +657,33 @@ BaseJob::duration_ms_t BaseJob::millisToRetry() const
     return timeToRetry().count();
 }
 
-int BaseJob::maxRetries() const { return d->maxRetries; }
+int BaseJob::maxRetries() const
+{
+    return d->backoffStrategy.maxRetries ? static_cast<int>(*d->backoffStrategy.maxRetries)
+                                         : std::numeric_limits<int>::max();
+}
 
 void BaseJob::setMaxRetries(int newMaxRetries)
 {
-    d->maxRetries = newMaxRetries;
+    d->backoffStrategy.maxRetries = newMaxRetries;
+}
+
+JobBackoffStrategy BaseJob::currentBackoffStrategy() const { return d->backoffStrategy; }
+
+void BaseJob::setBackoffStrategy(JobBackoffStrategy strategy)
+{
+    QUO_CHECK(!strategy.jobTimeouts.empty());
+    QUO_CHECK(!strategy.nextRetryIntervals.empty());
+    d->backoffStrategy = std::move(strategy);
+}
+
+JobBackoffStrategy BaseJob::defaultBackoffStrategy() { return Private::defaultBackoffStrategy; }
+
+void BaseJob::setDefaultBackoffStrategy(JobBackoffStrategy defaultStrategy)
+{
+    QUO_CHECK(!defaultStrategy.jobTimeouts.empty());
+    QUO_CHECK(!defaultStrategy.nextRetryIntervals.empty());
+    Private::defaultBackoffStrategy = std::move(defaultStrategy);
 }
 
 BaseJob::Status BaseJob::status() const { return d->status; }
@@ -775,7 +789,7 @@ void BaseJob::setStatus(Status s)
 
     if (!s.message.isEmpty() && d->connection
         && !d->connection->accessToken().isEmpty())
-        s.message.replace(QString::fromUtf8(d->connection->accessToken()), "(REDACTED)"_ls);
+        s.message.replace(QString::fromUtf8(d->connection->accessToken()), "(REDACTED)"_L1);
     if (!s.good())
         qCWarning(d->logCat) << this << "status" << s;
     d->status = std::move(s);
@@ -787,6 +801,13 @@ void BaseJob::setStatus(int code, QString message)
     setStatus({ code, std::move(message) });
 }
 
+void BaseJob::forceResult(QJsonDocument resultDoc, Status s)
+{
+    d->jsonResponse = std::move(resultDoc);
+    setStatus(std::move(s));
+    QMetaObject::invokeMethod(this, [this] { finishJob(); }, Qt::QueuedConnection);
+}
+
 void BaseJob::abandon()
 {
     beforeAbandon();
@@ -796,14 +817,21 @@ void BaseJob::abandon()
     if (d->reply)
         d->reply->disconnect(this);
     emit finished(this);
+    if (QLibraryInfo::version() < QVersionNumber(6, 5))
+        future().cancel(); // Qt 6.4 didn't do it on the promise destruction, see QTBUG-103992
 
-    deleteLater();
+    deleteLater(); // The promise will cancel itself on deletion
 }
 
 void BaseJob::timeout()
 {
-    setStatus(Timeout, "The job has timed out"_ls);
+    setStatus(Timeout, "The job has timed out"_L1);
     finishJob();
 }
 
-void BaseJob::setLoggingCategory(LoggingCategory lcf) { d->logCat = lcf; }
+void BaseJob::setLoggingCategory(QMessageLogger::CategoryFunction lcf)
+{
+    d->logCat = lcf;
+}
+
+QFuture<void> BaseJob::future() { return d->promise.future(); }
